#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static bool entry_impl(const char **const ptr, char **const out) {
  char *const savepoint = out ? *out : nullptr;
  bool success = true;
  while (**ptr && **ptr != ')') {
    switch (**ptr) {
    case '\\': (*ptr)++;
    default:
      const char c = **ptr;
      if (*++*ptr == '*')
        (*ptr)++;
      else if (out)
        *(*out)++ = c;
      break;
    case '[':
      (*ptr) += 2;
      if (**ptr == '*')
        (*ptr)++;
      else
        success = false;
      break;
    case '(': {
      (*ptr)++;
      char *const savepoint = out ? *out : nullptr;
      bool inner_success = entry_impl(ptr, out);
      (*ptr)++;
      if (!inner_success) {
        if (**ptr == '*') {
          (*ptr)++;
          if (out) *out = savepoint;
        } else
          success = false;
      }
      break;
    }
    case '|':
      (*ptr)++;
      if (success || !out) {
        entry_impl(ptr, nullptr);
        break;
      } else {
        *out = savepoint;
        return entry_impl(ptr, out);
      }
    }
  }
  return success;
}

bool entry(const char *regex, char *out) {
  if (entry_impl(&regex, &out)) {
    *out = 0;
    return true;
  }
  return false;
}

int run(char *regex, size_t len) {
  char *res = malloc(len);
  if (entry(regex, res)) {
    puts(res);
    free(res);
    return 0;
  } else {
    fputs("No valid string", stderr);
    free(res);
    return 1;
  }
}

int main(int argc, char **argv) {
  int ret;
  switch (argc) {
  case 1:
    const int INPUT_LIMIT = 1'000;
    char * const input = malloc(INPUT_LIMIT);
    if (!fgets(input, INPUT_LIMIT, stdin)) {
      puts("Couldn't read input");
      return 1;
    }
    ret = run(input, strlen(input) + 1);
    free(input);
    return ret;
  case 2:
    size_t buffer_capacity = 2048;
    size_t buffer_size = 0;
    char *buffer = malloc(buffer_capacity);

    FILE *file;
    if (!(file = fopen(argv[1], "r"))) {
      fprintf(stderr, "Failed to read from %s", argv[1]);
      return 1;
    }
    size_t read_count;
    while (1) {
      if (buffer_size == buffer_capacity - 1) buffer = realloc(buffer, buffer_capacity *= 2);
      read_count = fread(buffer + buffer_size, 1, buffer_capacity - 1 - buffer_size, file);
      if (!read_count) break;
      buffer_size += read_count;
    }
    buffer[buffer_size] = 0;
    ret = run(buffer, buffer_size + 1);
    free(buffer);
    return ret;
  case 3: return run(argv[2], strlen(argv[2]) + 1);
  }
}